@@ -11,4 +11,6 @@ fn tests() {
     t.pass("tests/07-mod");
     t.compile_fail("tests/08-wrong-args");
     t.compile_fail("tests/09-version-wrong");
+    t.pass("tests/10-declare-janet-module");
+    t.compile_fail("tests/11-declare_mod_errors");
 }
